#!/usr/bin/env perl

use strict;
use warnings;


use Cwd qw(cwd getcwd);

if (scalar(@ARGV) != 3) {
    print STDERR "****\n";
    print STDERR "* Error: incorrect usage\n";
    print STDERR "* Usage: $0 frame-step-in-secs frame-duration-in-secs audio_input_file\n";
    print STDERR "****\n";
    exit(1);
}

my $frame_step_secs = $ARGV[0];
my $frame_dur_secs  = $ARGV[1];

my $audio_file        = $ARGV[2];
my ($full_audio_root) = ($audio_file =~ m/^(.*)\..+?$/);
my ($audio_root)      = ($full_audio_root =~ m/^.*\/(.*?)$/);

if ( ! -d "tmp" ) {
    print "Creating directory: tmp\n";
    mkdir("tmp");
}

my $tmp_json_file = "tmp/${audio_root}_essentiafeatures.json";
my $tmp_csv_file  = "tmp/${audio_root}_essentiafeatures.csv";

my $csv_file  = "${full_audio_root}_essentiafeatures_frames.csv";

my $pwd = cwd();
my $profile_template_file = "$pwd/essentia-2013-2014.profile.in";
my $profile_file = "$pwd/essentia-2013-2014.profile";

# knock out any arrays in the JSON extracted features file
my $ignore_fields="\
 lowlevel.barkbands.* \
 lowlevel.erbbands.* \
 lowlevel.gfcc.* \
 lowlevel.melbands.* \
 lowlevel.melbands128.* \
 lowlevel.mfcc.* \
\
 lowlevel.spectral_contrast_coeffs.* \
 lowlevel.spectral_contrast_valleys.* \
\
 metadata.* \
\
 rhythm.beats_loudness_band_ratio.* \
 rhythm.beats_position.* \
 rhythm.bpm_histogram.* \
\
 tonal.hpcp.* \
 tonal.chords_histogram.* \
 tonal.thpcp.*";

$ignore_fields =~ s/\n//sg;


my $audio_cmd = "./AUDIO-DURATION.sh $audio_file";

open(my $fh, '-|', $audio_cmd) or die $!;

# Just need to read in one line:

my $audio_duration = <$fh>;
chomp($audio_duration);

# otherwise would do
#
#while (my $line = <$fh>) {
#    # Do stuff with each $line.
#}

close($fh);


    
if ( ! -f $csv_file ) {
       
    print "******\n";
    print "* Running Essentia music extractor\n";
    print "*  on input file:     $audio_file (duration $audio_duration)\n";
    print "*  with profile:      $profile_file\n";
    print "*  generating output: $csv_file\n";
    print "****\n";

    open(my $ofh, '>', $csv_file) or die $!;


    for (my $t=0; $t<$audio_duration; $t+=$frame_step_secs) {
	##print "," if ($t>0);
	print "*\n";
	print "*\n";
	print "* ### [Time step: $t]\n";
	print "*\n";
	print "*\n";

	#
	# Generate bespoke profile_file for this time-slice
	#
	open(my $ipfh, '<', $profile_template_file) or die $!;
	open(my $opfh, '>', $profile_file) or die $!;

	my $startTime = $t;
	my $endTime   = $t + $frame_dur_secs;
	
	while (my $line = <$ipfh>) {
	    chomp($line);
	    $line =~ s/\@startTime\@/$startTime/g;
	    $line =~ s/\@endTime\@/$endTime/g;

	    print $opfh "$line\n";
	}

	close($opfh);
	close($ipfh);

	#
	# Extract features and convert to CSV
	#

	my $extract_cmd = "essentia_streaming_extractor_music $audio_file $tmp_json_file $profile_file";
	my $extract_status = system($extract_cmd);
	if ($extract_status != 0) {
	    print STDERR "$!\n";
	    print STDERR "Error: Failed to run command:\n";
	    print STDERR "    $extract_cmd\n";
	    next;
	}

	my $convert_cmd = "json_to_csv.py -i $tmp_json_file -o $tmp_csv_file --ignore $ignore_fields";
	my $convert_status = system($convert_cmd);
	if ($convert_status != 0) {
	    print STDERR "$!\n";
	    print STDERR "Error: Failed to run command:\n";
	    print STDERR "    $convert_cmd\n";
	    next;
	}

	open(my $ifh, '<', $tmp_csv_file) or die $!;

	# read in lines to array: single line shot
	chomp(my @lines = <$ifh>);
	
	# my @lines = ();
	#
	# while (my $line = <$ifh>) {
	#     chomp($line);
	#     push(@lines,$line);
	# }

	close($ifh);
	
	if ($t == 0) {
	    # output first line from $tmp_csv_file to $csv_file
	    print $ofh $lines[0], "\n";
	}

	# append 2nd line of $tmp_json_file (i.e. data vals) to $csv_file
	print $ofh $lines[1], "\n";
		
	unlink($tmp_json_file) or die "Can't delete $tmp_json_file: $!\n";
	unlink($tmp_csv_file)  or die "Can't delete $tmp_csv_file: $!\n";
	
	# break out of loop if there isn't enough time left for a full $frame_dur_secs
	my $end_of_next_frame = $t+$frame_step_secs+$frame_dur_secs;	
	last if ($end_of_next_frame > $audio_duration);
    }

    close($ofh);
    
    print "******\n";
    
}
else {
    print "*  Skipping frame-by-frame audio features computation as $csv_file already exists\n";
}



